@@ -36,7 +36,10 @@ func TestSubjectTreeBasics(t *testing.T) {
 	require_True(t, old == nil)
 	require_False(t, updated)
 	require_Equal(t, st.Size(), 1)
-	// Find with single leaf.
+	// Find shouldn't work with a wildcard.
+	_, found := st.Find(b("foo.bar.*"))
+	require_False(t, found)
+	// But it should with a literal. Find with single leaf.
 	v, found := st.Find(b("foo.bar.baz"))
 	require_True(t, found)
 	require_Equal(t, *v, 22)
@@ -78,7 +81,6 @@ func TestSubjectTreeNodeGrow(t *testing.T) {
 	require_False(t, updated)
 	_, ok = st.root.(*node16)
 	require_True(t, ok)
-	// We do not have node48, so once we fill this we should jump to node256.
 	for i := 5; i < 16; i++ {
 		subj := b(fmt.Sprintf("foo.bar.%c", 'A'+i))
 		old, updated := st.Insert(subj, 22)
@@ -89,6 +91,20 @@ func TestSubjectTreeNodeGrow(t *testing.T) {
 	old, updated = st.Insert(b("foo.bar.Q"), 22)
 	require_True(t, old == nil)
 	require_False(t, updated)
+	_, ok = st.root.(*node48)
+	require_True(t, ok)
+	// Fill the node48.
+	for i := 17; i < 48; i++ {
+		subj := b(fmt.Sprintf("foo.bar.%c", 'A'+i))
+		old, updated := st.Insert(subj, 22)
+		require_True(t, old == nil)
+		require_False(t, updated)
+	}
+	// This one will trigger us to grow.
+	subj := b(fmt.Sprintf("foo.bar.%c", 'A'+49))
+	old, updated = st.Insert(subj, 22)
+	require_True(t, old == nil)
+	require_False(t, updated)
 	_, ok = st.root.(*node256)
 	require_True(t, ok)
 }
@@ -160,18 +176,34 @@ func TestSubjectTreeNodeDelete(t *testing.T) {
 	require_Equal(t, *v, 22)
 	_, ok = st.root.(*node4)
 	require_True(t, ok)
-	// Now pop up to node256
+	// Now pop up to node48
 	st = NewSubjectTree[int]()
 	for i := 0; i < 17; i++ {
 		subj := fmt.Sprintf("foo.bar.%c", 'A'+i)
 		st.Insert(b(subj), 22)
 	}
+	_, ok = st.root.(*node48)
+	require_True(t, ok)
+	v, found = st.Delete(b("foo.bar.A"))
+	require_True(t, found)
+	require_Equal(t, *v, 22)
+	_, ok = st.root.(*node16)
+	require_True(t, ok)
+	v, found = st.Find(b("foo.bar.B"))
+	require_True(t, found)
+	require_Equal(t, *v, 22)
+	// Now pop up to node256
+	st = NewSubjectTree[int]()
+	for i := 0; i < 49; i++ {
+		subj := fmt.Sprintf("foo.bar.%c", 'A'+i)
+		st.Insert(b(subj), 22)
+	}
 	_, ok = st.root.(*node256)
 	require_True(t, ok)
 	v, found = st.Delete(b("foo.bar.A"))
 	require_True(t, found)
 	require_Equal(t, *v, 22)
-	_, ok = st.root.(*node16)
+	_, ok = st.root.(*node48)
 	require_True(t, ok)
 	v, found = st.Find(b("foo.bar.B"))
 	require_True(t, found)
@@ -341,7 +373,7 @@ func TestSubjectTreeNoPrefix(t *testing.T) {
 		require_True(t, old == nil)
 		require_False(t, updated)
 	}
-	n, ok := st.root.(*node256)
+	n, ok := st.root.(*node48)
 	require_True(t, ok)
 	require_Equal(t, n.numChildren(), 26)
 	v, found := st.Delete(b("B"))
@@ -636,3 +668,110 @@ func TestSubjectTreeMatchAllPerf(t *testing.T) {
 		t.Logf("Match %q took %s and matched %d entries", f, time.Since(start), count)
 	}
 }
+
+func TestSubjectTreeNode48(t *testing.T) {
+	var a, b, c leaf[int]
+	var n node48
+
+	n.addChild('A', &a)
+	require_Equal(t, n.key['A'], 1)
+	require_True(t, n.child[0] != nil)
+	require_Equal(t, n.child[0].(*leaf[int]), &a)
+	require_Equal(t, len(n.children()), 1)
+
+	child := n.findChild('A')
+	require_True(t, child != nil)
+	require_Equal(t, (*child).(*leaf[int]), &a)
+
+	n.addChild('B', &b)
+	require_Equal(t, n.key['B'], 2)
+	require_True(t, n.child[1] != nil)
+	require_Equal(t, n.child[1].(*leaf[int]), &b)
+	require_Equal(t, len(n.children()), 2)
+
+	child = n.findChild('B')
+	require_True(t, child != nil)
+	require_Equal(t, (*child).(*leaf[int]), &b)
+
+	n.addChild('C', &c)
+	require_Equal(t, n.key['C'], 3)
+	require_True(t, n.child[2] != nil)
+	require_Equal(t, n.child[2].(*leaf[int]), &c)
+	require_Equal(t, len(n.children()), 3)
+
+	child = n.findChild('C')
+	require_True(t, child != nil)
+	require_Equal(t, (*child).(*leaf[int]), &c)
+
+	n.deleteChild('A')
+	require_Equal(t, len(n.children()), 2)
+	require_Equal(t, n.key['A'], 0) // Now deleted
+	require_Equal(t, n.key['B'], 2) // Untouched
+	require_Equal(t, n.key['C'], 1) // Where A was
+
+	child = n.findChild('A')
+	require_Equal(t, child, nil)
+	require_True(t, n.child[0] != nil)
+	require_Equal(t, n.child[0].(*leaf[int]), &c)
+
+	child = n.findChild('B')
+	require_True(t, child != nil)
+	require_Equal(t, (*child).(*leaf[int]), &b)
+	require_True(t, n.child[1] != nil)
+	require_Equal(t, n.child[1].(*leaf[int]), &b)
+
+	child = n.findChild('C')
+	require_True(t, child != nil)
+	require_Equal(t, (*child).(*leaf[int]), &c)
+	require_True(t, n.child[2] == nil)
+
+	var gotB, gotC bool
+	var iterations int
+	n.iter(func(n node) bool {
+		iterations++
+		if gb, ok := n.(*leaf[int]); ok && &b == gb {
+			gotB = true
+		}
+		if gc, ok := n.(*leaf[int]); ok && &c == gc {
+			gotC = true
+		}
+		return true
+	})
+	require_Equal(t, iterations, 2)
+	require_True(t, gotB)
+	require_True(t, gotC)
+}
+
+func TestSubjectTreeMatchNoCallbackDupe(t *testing.T) {
+	st := NewSubjectTree[int]()
+	st.Insert(b("foo.bar.A"), 1)
+	st.Insert(b("foo.bar.B"), 1)
+	st.Insert(b("foo.bar.C"), 1)
+	st.Insert(b("foo.bar.>"), 1)
+
+	for _, f := range [][]byte{
+		[]byte(">"),
+		[]byte("foo.>"),
+		[]byte("foo.bar.>"),
+	} {
+		seen := map[string]struct{}{}
+		st.Match(f, func(bsubj []byte, _ *int) {
+			subj := string(bsubj)
+			if _, ok := seen[subj]; ok {
+				t.Logf("Match callback was called twice for %q", subj)
+			}
+			seen[subj] = struct{}{}
+		})
+	}
+}
+
+func TestSubjectTreeNilNoPanic(t *testing.T) {
+	var st *SubjectTree[int]
+	st.Match([]byte("foo"), func(_ []byte, _ *int) {})
+	_, found := st.Find([]byte("foo"))
+	require_False(t, found)
+	_, found = st.Delete([]byte("foo"))
+	require_False(t, found)
+	_, found = st.Insert([]byte("foo"), 22)
+	require_False(t, found)
+}
